/*
 *	func.h
 *
 *	function prototype declarations
 */

#include "depend.h"
#include "string.h"

/* macros */
#ifdef	USESTRERROR
#define	Xstrerror		strerror
#else
#define	Xstrerror(n)		(char *)sys_errlist[n]
#endif

#ifdef	USESIGPMASK
#define	Xsigemptyset(m)		sigemptyset(&(m))
#define	Xsigfillset(m)		sigfillset(&(m))
#define	Xsigaddset(m,s)		sigaddset(&(m), s)
#define	Xsigdelset(m,s)		sigdelset(&(m), s)
#else	/* !USESIGPMASK */
#define	Xsigemptyset(m)		((m) = 0)
#define	Xsigfillset(m)		((m) = ~0)
#define	Xsigaddset(m,s)		((m) |= sigmask(s))
#define	Xsigdelset(m,s)		((m) &= ~sigmask(s))
#endif	/* !USESIGPMASK */

#ifdef	USEWAITPID
#define	Xwait3(s,o,r)		waitpid(-1, s, o)
#else
#define	Xwait3			wait3
#endif

#ifdef	GETPGRPVOID
#define	Xgetpgrp		getpgrp
#else
#define	Xgetpgrp()		getpgrp(0)
#endif

#if	(GETTODARGS == 1)
#define	Xgettimeofday(tv, tz)	gettimeofday(tv)
#else
#define	Xgettimeofday(tv, tz)	gettimeofday(tv, tz)
#endif

#ifdef	USEGETWD
#define	Xgetwd			getwd
#else
#define	Xgetwd(p)		getcwd(p, MAXPATHLEN)
#endif

#ifdef	USERAND48
#define	Xsrandom		srand48
#define	Xrandom			lrand48
#else
#define	Xsrandom		srandom
#define	Xrandom			random
#endif

#ifdef	CYGWIN
#define	strpathcmp		Xstrcasecmp
#define	strnpathcmp		Xstrncasecmp
#else
#define	strpathcmp		strcmp
#define	strnpathcmp		strncmp
#endif

#ifdef	ULONGIOCTL
typedef unsigned long		ioctlreq_t;
#else
typedef int			ioctlreq_t;
#endif

#define	base64siz(n)		((((n) - 1) / 3 + 1) * 4)

/* syscall.c */
extern int Xselect __P_((int, int, int));
extern int Xread __P_((int, char *, ALLOC_T, int, CONST char *));
extern int Xwrite __P_((int, CONST char *, ALLOC_T, int, CONST char *));
extern int Xopen __P_((CONST char *, int, int, int));
extern int Xclose __P_((int, CONST char *));
extern off_t Xlseek __P_((int, off_t, int, CONST char *));
extern int Xfcntl __P_((int, int, VOID_P, CONST char *));
#ifndef	NOFLOCK
extern int Xflock __P_((int, int, CONST char *));
#endif
extern int Xftruncate __P_((int, off_t, CONST char *));
extern int Xpipe __P_((int *));
extern int Xdup2 __P_((int, int, CONST char *));
extern int Xchdir __P_((CONST char *));
extern int Xmkdir __P_((CONST char *, int));
extern int Xrename __P_((CONST char *, CONST char *));
extern int Xunlink __P_((CONST char *));
extern int Xsetreuid __P_((uid_t, uid_t));
extern int Xsetregid __P_((gid_t, gid_t));
extern p_id_t Xsetsid __P_((VOID_A));
extern p_id_t Xfork __P_((VOID_A));
extern int Xsetpgrp __P_((p_id_t, p_id_t));
extern VOID Xexecve __P_((CONST char *, char *CONST [], char *CONST []));

/* libc.c */
extern VOID Xfree __P_((VOID_P));
extern char *Xmalloc __P_((ALLOC_T));
extern char *Xrealloc __P_((VOID_P, ALLOC_T));
extern char *Xstrdup __P_((CONST char *));
extern char *Xstrndup __P_((CONST char *, ALLOC_T));
extern int Xusleep __P_((u_long));
extern int Xdaemon __P_((int, int));

/* list.c */
extern int list_count __P_((char *CONST *));
extern char **list_init __P_((VOID));
extern char **list_add __P_((char **, int, CONST char *, ALLOC_T));
extern char **list_tryadd __P_((char **, char *));
extern VOID list_free __P_((char **));

/* time.c */
time_t Xtime __P_((time_t *));
int time_getdatestr __P_((char *, ALLOC_T));

/* term.c */
extern int Xioctl __P_((int, ioctlreq_t, VOID_P));
extern VOID term_safeclose __P_((int, CONST char *));
extern VOID term_loadtermio __P_((int, CONST char *, CONST char *));
extern VOID term_savetermio __P_((int, char **, char **));
extern VOID term_getterm __P_((char **, char **));
extern VOID term_closetty __P_((VOID_A));
extern int term_gettty __P_((VOID_A));
extern int term_hastty __P_((VOID_A));
extern int term_init __P_((VOID_A));

/* signal.c */
extern int sig_init __P_((VOID_A));
extern VOID sig_child __P_((VOID_A));
extern VOID sig_blockchild __P_((VOID_A));
extern VOID sig_unblockchild __P_((VOID_A));
extern VOID sig_push __P_((CONST char *));
extern VOID sig_pop __P_((CONST char *));
extern VOID sig_unlink __P_((VOID_A));
extern int sig_check __P_((int));

/* wait.c */
extern int wait_children __P_((VOID_A));
extern VOID wait_discard __P_((VOID_A));
extern int wait_push __P_((p_id_t));
extern int wait_pop __P_((p_id_t, int *));
extern VOID wait_cancel __P_((p_id_t));
extern int wait_child __P_((p_id_t, CONST char *, int));
extern VOID wait_killall __P_((int));
extern int wait_all __P_((int));

/* md5.c */
extern VOID md5_encode __P_((u_char *, ALLOC_T *, CONST u_char *, ALLOC_T));
extern VOID md5_hmac __P_((u_char *, ALLOC_T *, CONST char *, CONST char *));
extern int md5_fencode __P_((u_char *, ALLOC_T *, VOID_P,
	int (*)__P_((char *, ALLOC_T, VOID_P))));

/* base64.c */
extern int base64_encode __P_((char *, ALLOC_T, CONST u_char *, ALLOC_T));
extern int base64_decode __P_((u_char *, ALLOC_T *, CONST char *, ALLOC_T));

/* passwd.c */
extern int passwd_encode __P_((char *, ALLOC_T, CONST char *, int));
extern int passwd_decode __P_((char *, ALLOC_T, CONST char *));

/* pid.c */
extern int pid_check __P_((VOID_A));
extern int pid_create __P_((VOID_A));
extern VOID pid_unlink __P_((VOID_A));

/* log.c */
extern VOID log_init __P_((VOID_A));
extern VOID log_end __P_((VOID_A));
extern VOID log_header __P_((CONST char *, CONST char *, int, int));
extern VOID log_body __P_((CONST char *, ...));
extern char *log_strerror __P_((VOID_A));
extern VOID log_debugfp __P_((int, CONST char *, XFILE *));

/* config.c */
extern VOID conf_init __P_((int));
extern char *conf_getversion __P_((int *));
extern int conf_args __P_((int, char *CONST []));
extern char *conf_getline __P_((XFILE *, int));
extern int conf_load __P_((CONST char *, int));
extern VOID conf_push __P_((VOID_A));
extern VOID conf_pop __P_((VOID_A));

/* socket.c */
extern int sock_changeopt __P_((int, int));
extern VOID Xshutdown __P_((int, int));
extern int sock_issocket __P_((int));
extern int sock_wait __P_((int));
extern char *sock_getline __P_((XFILE *, ALLOC_T *));

/* stat.c */
extern int stat_getsize __P_((CONST char *, ALLOC_T *, int));
extern int stat_gettime __P_((CONST char *, time_t *, int));
extern int stat_getmode __P_((CONST char *, u_int *, int));
extern int stat_isdir __P_((CONST char *, int));
extern int stat_isreg __P_((CONST char *, int));
extern int stat_cloexec __P_((int, CONST char *));

/* filesys.c */
extern char *fsys_getmnttype __P_((CONST char *));
extern int fsys_isnfs __P_((CONST char *));

/* pwd.c */
extern char *pwd_gethome __P_((uid_t));
extern char *pwd_getpass __P_((CONST char *));
extern int pwd_login __P_((CONST char *));
extern char *pwd_gethost __P_((VOID_A));

/* pty.c */
extern int Xopenpty __P_((int *, char *, ALLOC_T));
extern int Xlogin_tty __P_((CONST char *, CONST char *, CONST char *));
extern p_id_t Xforkpty __P_((int *, CONST char *, CONST char *));

/* wild.c */
extern u_short *wild_init __P_((CONST char *, ALLOC_T));
extern int wild_match __P_((CONST char *, ALLOC_T, CONST u_short *));

/* argv.c */
extern char **argv_getargv __P_((CONST char *, ALLOC_T));

/* dirent.c */
extern int dir_getlist __P_((CONST char *, int **));
extern int dir_atoi __P_((CONST char *, char *CONST *, int **));

/* mhcmd.c */
extern int mh_genpath __P_((char *, ALLOC_T, CONST char *, int));
extern int mh_getseq __P_((CONST char *));
extern int mh_setseq __P_((CONST char *, int));
extern XFILE *mh_opendraft __P_((char *, ALLOC_T));
extern int mh_inc __P_((VOID_A));
extern int mh_pick __P_((CONST char *, char **, char ***, int));
extern int mh_rmm __P_((CONST char *, char **));
extern int mh_send __P_((CONST char *));

/* message.c */
extern char *msg_getline __P_((XFILE *, ALLOC_T *, int));
extern int msg_putline __P_((CONST char *, XFILE *));
extern char *msg_getfield __P_((char *, ALLOC_T *, int));
extern int msg_send __P_((XFILE *, CONST char *, ...));
extern int msg_sendeom __P_((XFILE *));
extern char **msg_getargv __P_((CONST char *, ALLOC_T));
extern VOID msg_hex __P_((char *, ALLOC_T, CONST u_char *, ALLOC_T));

/* auth.c */
extern int auth_cmppasswd __P_((CONST char *, CONST char *));
extern int auth_getmaxid __P_((VOID_A));
extern int auth_getid __P_((CONST char *, ALLOC_T));
extern int auth_genlist __P_((char *, ALLOC_T));
extern int auth_setmech __P_((char *CONST *, XFILE *));
extern int auth_input __P_((CONST char *, XFILE *));
